#!/usr/bin/env ruby
##
# Copyright (c) 2013 Christian Ferraz Lemos de Sousa
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is furnished to do
# so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
# CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
# OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
####

# Bunch of function redefinition to make use
# of Ruby 2.0 new standard. Also, require this
# file to enforce the use of Ruby 2.0.

# ensure that we have monk
require 'sinatra-monk/base'

module Sinatra
  module Monk
    # Provides a base class for Mongo database management. This is the simplest way
    # to connect to Mongo with Monk. It is intented to keep Sinatra's philosofy of
    # keeping it as simple as possible for use.
    class MBase
      # Aliased initialize method, made to add Ruby 2.0 version.
      alias _ol_initialize initialize

      # (Ruby 2.x) Setup the Monk with the data that was sent.
      # Autoconnect if :connect flag is true.
      # @param user [String] the database username,
      # @param pass [String] the password of the previous database user,
      # @param database [String] the name of the database,
      # @param host [String] the host of the database,
      # @param port [Fixnum] the port to connect to the database,
      # @param opts [Hash] monk options (set :conect to autoconnect the Monk)
      def initialize(user:'', pass:'', database:'local', host:'localhost',
                     port:27017, opts:{:connect => false})
        _ol_initialize(user, pass, database, host, port, opts)
      end
    end

    # A type of Monk that acts like a Hash, it was made to look very native.
    # It still need some improvement.
    class MHash
      # Aliased initialize method, made to add Ruby 2.0 version.
      alias _ol_initialize initialize
      # (Ruby 2.x) Differently from MBase, MHash enforce autoconnect.
      # @param name [String] name of the used/created collection,
      # @param user [String] the database username,
      # @param pass [String] the password of the previous database user,
      # @param database [String] the name of the database,
      # @param host [String] the host of the database,
      # @param port [Fixnum] the port to connect to the database,
      # @param opts [Hash] monk options (set :conect to autoconnect the Monk)
      def initialize(name:'monk', user:'', pass:'', database:'local', host:'localhost',
                     port:27017, opts:{:connect => false})
        _ol_initialize(name, user, pass, database, host, port, opts)
      end
    end

    # (Ruby 2.x) A helper that create a MBase instance.
    # @param user [String] the database username,
    # @param pass [String] the password of the previous database user,
    # @param database [String] the name of the database,
    # @param host [String] the host of the database,
    # @param port [Fixnum] the port to connect to the database,
    # @param opts [Hash] monk options (set :conect to autoconnect the Monk)
    # @return [MBase] the Monk instance.
    def monk(user:'', pass:'', database:'local', host:'localhost',
             port:27017, opts:{:connect => false})
      return MBase.new(user, pass, database, host, port, opts)
    end

    # (Ruby 2.x) A helper that create a MHash instance.
    # @param name [String] name of the used/created collection,
    # @param user [String] the database username,
    # @param pass [String] the password of the previous database user,
    # @param database [String] the name of the database,
    # @param host [String] the host of the database,
    # @param port [Fixnum] the port to connect to the database,
    # @param opts [Hash] monk options (set :save to autosave after each change)
    # @return [MHash] the Monk instance.
    def hmonk(name:'monk', user:'', pass:'', database:'local', host:'localhost',
              port:27017, opts:{:connect => false})
      return MHash.new(name, user, pass, database, host, port, opts)
    end
  end
end
