#!/usr/bin/env ruby
##
# Copyright (c) 2013 Christian Ferraz Lemos de Sousa
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is furnished to do
# so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
# CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
# OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
####
# Some helpers to make life easier on Sinatra.
##

# helpers need sinatra
require 'sinatra-monk/base'
require 'sinatra/base'

module Sinatra
  module Monk
    # (Any Ruby) A helper that create a MBase instance.
    # @param user [String] the database username,
    # @param pass [String] the password of the previous database user,
    # @param database [String] the name of the database,
    # @param host [String] the host of the database,
    # @param port [Fixnum] the port to connect to the database,
    # @param opts [Hash] monk options (set :conect to autoconnect the Monk)
    # @return [MBase] the Monk instance.
    def monk user = '', pass = '', database = 'local', host = 'localhost',
             port = 27017, opts = {:connect => false}
      return MBase.new(user, pass, database, host, port, opts)
    end

    # (Any Ruby) A helper that create a MHash instance.
    # @param name [String] name of the used/created collection,
    # @param user [String] the database username,
    # @param pass [String] the password of the previous database user,
    # @param database [String] the name of the database,
    # @param host [String] the host of the database,
    # @param port [Fixnum] the port to connect to the database,
    # @param opts [Hash] monk options (set :save to autosave after each change)
    # @return [MHash] the Monk instance.
    def hmonk name = 'monk', user = '', pass = '', database = 'local',
              host = 'localhost', port = 27017, opts = {:save => true}
      return MHash.new(name, user, pass, database, host, port, opts)
    end
  end
  helpers Monk
end
